#include "stdafx.h"
#include "Functions_OS.h"
#include "commons.h"

BOOL Time1970ToSystemTime(DWORD toffset, SYSTEMTIME *sys_time)
{
	SYSTEMTIME age_time_s;
	FILETIME age_time_f;
	ULARGE_INTEGER nanosec_time, nanosec_offset;

	memset(&age_time_s, 0, sizeof(age_time_s));
	age_time_s.wDay = 1;
	age_time_s.wMonth = 1;
	age_time_s.wYear = 1970;
	if (!SystemTimeToFileTime(&age_time_s, &age_time_f))
		return FALSE;

	nanosec_offset.QuadPart = (unsigned int)toffset;
	nanosec_offset.QuadPart *= 10000000;

	nanosec_time.HighPart = age_time_f.dwHighDateTime;
	nanosec_time.LowPart = age_time_f.dwLowDateTime;
	nanosec_time.QuadPart += nanosec_offset.QuadPart;

	age_time_f.dwHighDateTime = nanosec_time.HighPart;
	age_time_f.dwLowDateTime = nanosec_time.LowPart;

	return FileTimeToSystemTime(&age_time_f, sys_time);
}

DWORD GetOsImage(os_struct_t *os_info)
{
	if (os_info->is_supported)
		return 4;
	return 5;
}

DWORD GetArch(os_struct_t *os_info)
{
	WCHAR wow64_path[MAX_PATH];
	HANDLE hfile;

	swprintf_s(wow64_path, MAX_PATH, L"%s%s", os_info->drive, L"windows\\syswow64\\");
	hfile = CreateFile(wow64_path, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, NULL);
	if (hfile != INVALID_HANDLE_VALUE) {
		CloseHandle(hfile);
		return 64;
	}

	return 32;
}

BOOL IsSupported(os_struct_t *os_info)
{
	if (os_info->arch == 64)
		return FALSE;

	if ( !wcscmp(os_info->curr_ver, L"5.0") || // Windows 2000
		 !wcscmp(os_info->curr_ver, L"5.1") || // Windows XP
		 !wcscmp(os_info->curr_ver, L"5.2") || // Windows 2003
		 !wcscmp(os_info->curr_ver, L"6.0"))   // Windows Vista o 2008
		return TRUE;

	return FALSE;
}

os_struct_t *GetOSList() 
{
	os_struct_t *list_head = NULL;
	os_struct_t **list_curr = NULL;
	WCHAR driver_letters[512];
	DWORD driver_len, i;
	WCHAR software_hive_path[MAX_PATH];
	WCHAR system_hive_path[MAX_PATH];
	WCHAR system_root[MAX_PATH];
	HANDLE hfile;
	DWORD *temp_date;

	if (!(driver_len = GetLogicalDriveStrings(sizeof(driver_letters)/sizeof(driver_letters[0]), driver_letters)))
		return NULL;

	list_curr = &list_head;
	// Cicla tutti i volumi montati
	for (i=0; i<driver_len; i+=4) {
		// Vede se esiste l'hive (XP, Vista, 2000, etc.)
		// Cerca prima in \Windows e poi in \WinNT
		// Deve trovare sia SYSTEM che SOFTWARE per proseguire
		swprintf_s(system_root, MAX_PATH, L"windows\\system32");
		swprintf_s(system_hive_path, MAX_PATH, L"%s%s%s", &driver_letters[i], system_root, L"\\config\\system");
		
		hfile = CreateFile(system_hive_path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL);
		if (hfile == INVALID_HANDLE_VALUE) {
			swprintf_s(system_root, MAX_PATH, L"winnt\\system32");
			swprintf_s(system_hive_path, MAX_PATH, L"%s%s%s", &driver_letters[i], system_root, L"\\config\\system");
			hfile = CreateFile(system_hive_path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL);
			if (hfile == INVALID_HANDLE_VALUE)
				continue;
		}
		CloseHandle(hfile);
		swprintf_s(software_hive_path, MAX_PATH, L"%s%s%s", &driver_letters[i], system_root, L"\\config\\software");
		hfile = CreateFile(software_hive_path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL);
		if (hfile == INVALID_HANDLE_VALUE)
			continue;
		CloseHandle(hfile);


		// Monta l'hive
		if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\", system_hive_path) != ERROR_SUCCESS) 
			continue;

		if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SOFTWARE\\", software_hive_path) != ERROR_SUCCESS) {
			RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\");
			continue;
		}

		do {
			// Crea la struttura
			if ( !(*list_curr = (os_struct_t *)malloc(sizeof(os_struct_t))) ) 
				break;

			// Valorizza la struttura
			memset((*list_curr), 0, sizeof(os_struct_t));
			memcpy((*list_curr)->drive, &driver_letters[i], 4*sizeof(WCHAR));
			memcpy((*list_curr)->system_root, system_root, sizeof(system_root));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"ProductName", NULL, &((*list_curr)->product_name));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"CSDVersion", NULL, &((*list_curr)->csd_version));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"RegisteredOwner", NULL, &((*list_curr)->reg_owner));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"RegisteredOrganization", NULL, &((*list_curr)->reg_org));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"ProductId", NULL, &((*list_curr)->product_id));
			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"CurrentVersion", NULL, &((*list_curr)->curr_ver));

			ReadRegValue(L"RCS_SYSTEM\\ControlSet001\\Control\\ProductOptions", L"ProductType", NULL, &((*list_curr)->product_type));
			ReadRegValue(L"RCS_SYSTEM\\ControlSet001\\Control\\ComputerName\\ComputerName", L"ComputerName", NULL, &((*list_curr)->computer_name));

			ReadRegValue(L"RCS_SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", L"InstallDate", NULL, (WCHAR **)&temp_date);
			if (temp_date) {
				Time1970ToSystemTime(*temp_date, &((*list_curr)->install_date));
				SAFE_FREE(temp_date);
			}

			(*list_curr)->arch = GetArch(*list_curr);
			(*list_curr)->is_supported = IsSupported(*list_curr); // Va chiamata per ultima...

			list_curr = &((*list_curr)->next);
		} while(0);

		RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SOFTWARE\\");
		RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\");
	}

	return list_head;
}

void FreeOSList(os_struct_t *list_head)
{
	os_struct_t *next;

	while(list_head) {
		next = list_head->next;

		SAFE_FREE(list_head->product_type);
		SAFE_FREE(list_head->computer_name);
		SAFE_FREE(list_head->product_name);
		SAFE_FREE(list_head->csd_version);
		SAFE_FREE(list_head->reg_owner);
		SAFE_FREE(list_head->reg_org);
		SAFE_FREE(list_head->curr_ver);
		SAFE_FREE(list_head->product_id);

		SAFE_FREE(list_head);
		list_head = next;
	}
}

